use std::io::Write;
use std::net::TcpStream;
use crypto_box::{
    aead::{Aead, AeadCore, OsRng},
    SalsaBox, PublicKey, SecretKey
};

fn main() {
    //let stream = TcpStream::connect("172.30.16.1:8080");
    //let stream = TcpStream::connect("27.0.0.1:8080");
    match TcpStream::connect("localhost:7878") {
        Ok(mut stream) => {
            println!("Successfully connected to server");

            loop {
                let mut buffer = String::new();
                match std::io::stdin().read_line(&mut buffer) {
                    Ok(n) => {
                        if n == 0 {
                            break; // Beenden bei EOF
                        }
                        //
                        // Encryption
                        //

                        // Generate a random secret key.
                        // NOTE: The secret key bytes can be accessed by calling `secret_key.as_bytes()`
                        let alice_secret_key = SecretKey::generate(&mut OsRng);

                        // Get the public key for the secret key we just generated
                        let alice_public_key_bytes = alice_secret_key.public_key().as_bytes().clone();

                        // Obtain your recipient's public key.
                        let bob_public_key = PublicKey::from([
                            0xe8, 0x98, 0xc, 0x86, 0xe0, 0x32, 0xf1, 0xeb,
                            0x29, 0x75, 0x5, 0x2e, 0x8d, 0x65, 0xbd, 0xdd,
                            0x15, 0xc3, 0xb5, 0x96, 0x41, 0x17, 0x4e, 0xc9,
                            0x67, 0x8a, 0x53, 0x78, 0x9d, 0x92, 0xc7, 0x54,
                        ]);

                        // Create a `SalsaBox` by performing Diffie-Hellman key agreement between
                        // the two keys.
                        let alice_box = SalsaBox::new(&bob_public_key, &alice_secret_key);

                        // Get a random nonce to encrypt the message under
                        let nonce = SalsaBox::generate_nonce(&mut OsRng);

                        // Message to encrypt
                        let plaintext = buffer.trim().as_bytes();

                        // Encrypt the message using the box
                        let ciphertext = alice_box.encrypt(&nonce, &plaintext[..]).expect("Fehler");

                        println!("Sending {0} as {1:?}", buffer.trim(), plaintext);
                        stream.write(buffer.as_bytes()).unwrap();

                        //
                        // Decryption
                        //

                        // Either side can encrypt or decrypt messages under the Diffie-Hellman key
                        // they agree upon. The example below shows Bob's side.
                        let bob_secret_key = SecretKey::from([
                            0xb5, 0x81, 0xfb, 0x5a, 0xe1, 0x82, 0xa1, 0x6f,
                            0x60, 0x3f, 0x39, 0x27, 0xd, 0x4e, 0x3b, 0x95,
                            0xbc, 0x0, 0x83, 0x10, 0xb7, 0x27, 0xa1, 0x1d,
                            0xd4, 0xe7, 0x84, 0xa0, 0x4, 0x4d, 0x46, 0x1b
                        ]);

                        // Deserialize Alice's public key from bytes
                        let alice_public_key = PublicKey::from(alice_public_key_bytes);

                        // Bob can compute the same `SalsaBox` as Alice by performing the
                        // key agreement operation.
                        let bob_box = SalsaBox::new(&alice_public_key, &bob_secret_key);

                        // Decrypt the message, using the same randomly generated nonce
                        let decrypted_plaintext = bob_box.decrypt(&nonce, &ciphertext[..]).expect("Fehler");
                        let dec_plain_plaintext = std::str::from_utf8(&*decrypted_plaintext).expect("Nö");
                        assert_eq!(&plaintext[..], &decrypted_plaintext[..]);

                        println!("Sent {0:?} as cypher: {1:?}, decrypted: {2:?}, {3}", plaintext, ciphertext, decrypted_plaintext, dec_plain_plaintext);
                    }
                    Err(error) => {
                        println!("error: {error}");
                        break;
                    }
                }
            }
        }
        Err(e) => {
            println!("Failed to connect: {}", e);
        }
    }
    println!("Terminated.");
} // the stream is closed here
